package main

import (
	api "example.com/api"
	authdiscord "example.com/auth/discord"
	database "example.com/database"

	"github.com/gin-contrib/cors"
	"github.com/gin-gonic/gin"
)

func main() {
	api.GlobalDatabase = database.InitializeDatabase()
	api.GlobalConfig.ParseConfig("../config.toml")
	api.GlobalOAuth = authdiscord.CreateDiscordOAuthConfig(api.GlobalConfig)
	router := gin.Default()
	router.Use(cors.New(cors.Config{
		AllowOrigins:     []string{api.GlobalConfig.GetFrontendRootDomain()},
		AllowMethods:     []string{"PUT", "POST", "DELETE", "GET"},
		AllowHeaders:     []string{"Origin"},
		ExposeHeaders:    []string{"Content-Length"},
		AllowCredentials: true,
	}))
	// Authentication Workflow
	router.GET("/auth/callback", api.AuthCallback)
	router.GET("/auth/login", api.AuthLoginRedirect)
	router.GET("/auth/logout", api.AuthLogoutRedirect)
	// Create
	router.POST("/:object", api.CreateObject)
	// Update
	router.PUT("/:object", api.UpdateObject)
	// Read
	router.GET("/user/info", api.GetDiscordUser)
	router.GET("/user/authorized", api.GetUserLoggedIn)
	router.GET("/:object", api.GetObjects)
	router.GET("/all/:object", api.GetAllObjects)
	// Delete
	router.DELETE("/:object", api.DeleteObject)
	router.Run(":31337")
}
