package functiontag

import (
	"log"

	"github.com/gin-gonic/gin"
	"gorm.io/gorm"
)

type FunctionTag struct {
	gorm.Model
	Name string `gorm:"primaryKey uniqueIndex" json:"name"`
}

func (functionTag *FunctionTag) get(db *gorm.DB, inputFunctionTag string) {
	db.Where("name = ?", inputFunctionTag).Take(&functionTag)
}

func (functionTag FunctionTag) update(db *gorm.DB) error {
	result := db.Save(&functionTag)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (functionTag FunctionTag) create(db *gorm.DB) error {
	result := db.Create(&functionTag)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (functionTag FunctionTag) delete(db *gorm.DB) error {
	result := db.Delete(&functionTag)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func Create(db *gorm.DB, context *gin.Context) error {
	return FunctionTag{
		Name: context.Query("id"),
	}.create(db)
}

func Get(db *gorm.DB, inputFunctionTags []string) *[]FunctionTag {
	var outputFunctionTags []FunctionTag
	for _, inputFunctionTag := range inputFunctionTags {
		var outputFunctionTag FunctionTag
		outputFunctionTag.get(db, inputFunctionTag)
		outputFunctionTags = append(outputFunctionTags, outputFunctionTag)
	}
	return &outputFunctionTags
}

func GetAll(db *gorm.DB) *[]FunctionTag {
	var outputFunctionTags []FunctionTag
	var outputFunctionTagNames []string
	result := db.Model(&FunctionTag{}).Select("name").Find(&outputFunctionTagNames)
	if result.Error != nil {
		log.Println(result.Error)
	}
	outputFunctionTags = *Get(db, outputFunctionTagNames)
	return &outputFunctionTags
}

func Update(db *gorm.DB, context *gin.Context) error {
	return FunctionTag{
		Name: context.Query("id"),
	}.update(db)
}

func Delete(db *gorm.DB, inputFunctions []string) error {
	functions := Get(db, inputFunctions)
	for _, function := range *functions {
		err := function.delete(db)
		if err != nil {
			return err
		}
	}
	return nil
}
