package group

import (
	"log"

	"github.com/gin-gonic/gin"
	"gorm.io/gorm"
)

type Group struct {
	gorm.Model
	Name string `gorm:"primaryKey" json:"name"`
}

func (group *Group) Get(db *gorm.DB, inputGroup string) {
	db.Model(&Group{}).Where("name = ?", inputGroup).Take(&group)
}

func (group Group) update(db *gorm.DB) error {
	result := db.Save(&group)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (group Group) delete(db *gorm.DB) error {
	result := db.Delete(&group)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (group Group) Create(db *gorm.DB) error {
	result := db.Create(&group)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func Create(db *gorm.DB, context *gin.Context) error {
	return Group{
		Name: context.Query("id"),
	}.Create(db)
}

func Get(db *gorm.DB, inputGroups []string) *[]Group {
	var outputGroups []Group
	for _, inputGroup := range inputGroups {
		var outputGroup Group
		outputGroup.Get(db, inputGroup)
		outputGroups = append(outputGroups, outputGroup)
	}
	return &outputGroups
}

func GetAll(db *gorm.DB) *[]Group {
	var outputGroups []Group
	var outputGroupNames []string
	result := db.Model(&Group{}).Select("name").Find(&outputGroupNames)
	if result.Error != nil {
		log.Println(result.Error)
	}
	outputGroups = *Get(db, outputGroupNames)
	return &outputGroups
}

func Update(db *gorm.DB, context *gin.Context) error {
	return Group{
		Name: context.Query("id"),
	}.update(db)
}

func Delete(db *gorm.DB, inputGroups []string) error {
	groups := Get(db, inputGroups)
	for _, group := range *groups {
		err := group.delete(db)
		if err != nil {
			return err
		}
	}
	return nil
}
