package itemtag

import (
	"log"

	"gorm.io/gorm"
)

type ItemTag struct {
	gorm.Model
	Name string `gorm:"primaryKey uniqueIndex" json:"name"`
}

func (itemTag ItemTag) Create(db *gorm.DB) error {
	result := db.Create(&itemTag)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (itemTag *ItemTag) Get(db *gorm.DB, inputItemTag string) {
	db.Where("name = ?", inputItemTag).Take(&itemTag)
}

func (itemTag ItemTag) Update(db *gorm.DB) error {
	result := db.Save(&itemTag)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (itemTag ItemTag) Delete(db *gorm.DB) error {
	result := db.Delete(&itemTag)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func Create(db *gorm.DB, name string) error {
	return ItemTag{
		Name: name,
	}.Create(db)
}

func Get(db *gorm.DB, inputItemTags []string) *[]ItemTag {
	var outputItemTags []ItemTag
	for _, inputItemTag := range inputItemTags {
		var outputItemTag ItemTag
		outputItemTag.Get(db, inputItemTag)
		outputItemTags = append(outputItemTags, outputItemTag)
	}
	return &outputItemTags
}

func GetAll(db *gorm.DB) *[]ItemTag {
	var outputItemTagNames []string
	result := db.Model(&ItemTag{}).Select("name").Find(&outputItemTagNames)
	if result.Error != nil {
		log.Println(result.Error)
	}
	return Get(db, outputItemTagNames)
}

func Update(db *gorm.DB, name string) error {
	return ItemTag{
		Name: name,
	}.Update(db)
}

func Delete(db *gorm.DB, inputItemTags []string) {
	itemtags := Get(db, inputItemTags)
	for _, itemtag := range *itemtags {
		itemtag.Delete(db)
	}
}
