package person

import (
	"log"

	group "example.com/database/group"

	"gorm.io/gorm"
)

type Person struct {
	gorm.Model
	Name   string        `gorm:"primaryKey" json:"name"`
	Groups []group.Group `gorm:"many2many:person_group_associations" json:"groups"` // Unique
}

func (person Person) Create(db *gorm.DB) error {
	result := db.Create(&person)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (person *Person) getAssociations(db *gorm.DB) {
	db.Model(&person).Association("Groups").Find(&person.Groups)
}

func (person *Person) Get(db *gorm.DB, inputPerson string) {
	db.Where("name = ?", inputPerson).Take(&person)
	person.getAssociations(db)
}

func (person Person) Update(db *gorm.DB) error {
	var originalPerson Person
	originalPerson.Get(db, person.Name)
	groupsError := db.Model(&originalPerson).Association("Groups").Replace(&person.Groups)
	if groupsError != nil {
		return groupsError
	}
	return nil
}

func (person Person) Delete(db *gorm.DB) error {
	result := db.Delete(&person)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func Create(db *gorm.DB, name string, groups []string) error {
	return Person{
		Name:   name,
		Groups: *group.Get(db, groups),
	}.Create(db)
}

func Get(db *gorm.DB, inputPersons []string) *[]Person {
	var outputPersons []Person
	for _, inputPerson := range inputPersons {
		var outputPerson Person
		outputPerson.Get(db, inputPerson)
		outputPersons = append(outputPersons, outputPerson)
	}
	return &outputPersons
}

func GetAll(db *gorm.DB) *[]Person {
	var outputPersonNames []string
	result := db.Model(&Person{}).Select("name").Find(&outputPersonNames)
	if result.Error != nil {
		log.Println(result.Error)
	}
	return Get(db, outputPersonNames)
}

func Update(db *gorm.DB, name string, groups []string) error {
	return Person{
		Name:   name,
		Groups: *group.Get(db, groups),
	}.Update(db)
}

func Delete(db *gorm.DB, inputPersons []string) {
	persons := Get(db, inputPersons)
	for _, person := range *persons {
		person.Delete(db)
	}
}
