package role

import (
	"log"

	group "example.com/database/group"
	tier "example.com/database/tier"

	"gorm.io/gorm"
)

type Role struct {
	gorm.Model
	Name       string        `gorm:"primaryKey" json:"name"`
	Tiers      []tier.Tier   `gorm:"many2many:role_tier_associations" json:"tiers"`
	Visibility []group.Group `gorm:"many2many:role_visibility_associations" json:"visibility"` // Unique
}

func (role Role) Create(db *gorm.DB) error {
	result := db.Create(&role)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (role *Role) getAssociations(db *gorm.DB) {
	db.Model(&role).Association("Tiers").Find(&role.Tiers)
	db.Model(&role).Association("Visibility").Find(&role.Visibility)
	//
}

func (role *Role) Get(db *gorm.DB, inputRole string) {
	db.Where("name = ?", inputRole).Take(&role)
	role.getAssociations(db)
}

func (role Role) Update(db *gorm.DB) error {
	result := db.Save(&role)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (role Role) Delete(db *gorm.DB) error {
	result := db.Delete(&role)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func Create(db *gorm.DB, name string, tiers []int, visibility []string) error {
	return Role{
		Name:       name,
		Tiers:      *tier.Get(db, tiers),
		Visibility: *group.Get(db, visibility),
	}.Create(db)
}

func Get(db *gorm.DB, inputRoles []string) *[]Role {
	var outputRoles []Role
	for _, inputRole := range inputRoles {
		var outputRole Role
		outputRole.Get(db, inputRole)
		outputRoles = append(outputRoles, outputRole)
	}
	return &outputRoles
}

func GetAll(db *gorm.DB) *[]Role {
	var outputRoleNames []string
	result := db.Model(&Role{}).Select("name").Find(&outputRoleNames)
	if result.Error != nil {
		log.Println(result.Error)
	}
	return Get(db, outputRoleNames)
}

func Update(db *gorm.DB, name string, tiers []int, visibility []string) error {
	return Role{
		Name:       name,
		Tiers:      *tier.Get(db, tiers),
		Visibility: *group.Get(db, visibility),
	}.Update(db)
}

func Delete(db *gorm.DB, inputRoles []string) {
	roles := Get(db, inputRoles)
	for _, role := range *roles {
		role.Delete(db)
	}
}
