package schematic

import (
	"log"

	function "example.com/database/function"
	group "example.com/database/group"
	inventoryslot "example.com/database/inventoryslot"

	"gorm.io/gorm"
)

type Schematic struct {
	gorm.Model
	Material     []inventoryslot.InventorySlot `gorm:"many2many:schematic_material_associations" json:"material"`
	Tools        []inventoryslot.InventorySlot `gorm:"many2many:schematic_tool_associations" json:"tools"`
	Requirements []function.Function           `gorm:"many2many:schematic_requirement_associations" json:"requirements"`
	TimeUnits    int64                         `json:"time_units"` // Positive
	Result       inventoryslot.InventorySlot   `gorm:"many2many:schematic_result_associations" json:"result"`
	Visibility   []group.Group                 `gorm:"many2many:schematic_group_associations" json:"visibility"` // Unique
}

func (schematic Schematic) Create(db *gorm.DB) error {
	result := db.Create(&schematic)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (schematic *Schematic) getAssociations(db *gorm.DB) {
	db.Model(&schematic).Association("Material").Find(&schematic.Material)
	db.Model(&schematic).Association("Tools").Find(&schematic.Tools)
	db.Model(&schematic).Association("Requirements").Find(&schematic.Requirements)
	db.Model(&schematic).Association("Result").Find(&schematic.Result)
	db.Model(&schematic).Association("Visibility").Find(&schematic.Visibility)
}

func (schematic *Schematic) Get(db *gorm.DB, inputSchematic int) {
	db.Model(&Schematic{}).Where("id = ?", inputSchematic).Take(&schematic)
	schematic.getAssociations(db)
}

func (schematic Schematic) Update(db *gorm.DB) error {
	materialError := db.Model(&schematic).Association("Material").Replace(&schematic.Material)
	if materialError != nil {
		return materialError
	}
	toolsError := db.Model(&schematic).Association("Tools").Replace(&schematic.Tools)
	if toolsError != nil {
		return toolsError
	}
	requirementsError := db.Model(&schematic).Association("Requirements").Replace(&schematic.Requirements)
	if requirementsError != nil {
		return requirementsError
	}
	resultError := db.Model(&schematic).Association("Result").Replace(&schematic.Result)
	if resultError != nil {
		return resultError
	}
	visibilityError := db.Model(&schematic).Association("Visibility").Replace(&schematic.Visibility)
	if visibilityError != nil {
		return visibilityError
	}
	return nil
}

func (schematic Schematic) Delete(db *gorm.DB) error {
	result := db.Delete(&schematic)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func Create(db *gorm.DB, material []int, tools []int, requirements []string, timeUnits int64, result int, visibility []string) error {
	return Schematic{
		Material:     *inventoryslot.Get(db, material),
		Tools:        *inventoryslot.Get(db, tools),
		Requirements: *function.Get(db, requirements),
		TimeUnits:    timeUnits,
		Result:       (*inventoryslot.Get(db, []int{result}))[0],
		Visibility:   *group.Get(db, visibility),
	}.Create(db)
}

func Get(db *gorm.DB, inputSchematics []int) *[]Schematic {
	var outputSchematics []Schematic
	for _, inputSchematic := range inputSchematics {
		var outputSchematic Schematic
		outputSchematic.Get(db, inputSchematic)
		outputSchematics = append(outputSchematics, outputSchematic)
	}
	return &outputSchematics
}

func GetAll(db *gorm.DB) *[]Schematic {
	var outputSchematics []int
	result := db.Model(&Schematic{}).Select("id").Find(&outputSchematics)
	if result.Error != nil {
		log.Println(result.Error)
	}
	return Get(db, outputSchematics)
}

func Update(db *gorm.DB, material []int, tools []int, requirements []string, timeUnits int64, result int, visibility []string) error {
	return Schematic{
		Material:     *inventoryslot.Get(db, material),
		Tools:        *inventoryslot.Get(db, tools),
		Requirements: *function.Get(db, requirements),
		TimeUnits:    timeUnits,
		Result:       (*inventoryslot.Get(db, []int{result}))[0],
		Visibility:   *group.Get(db, visibility),
	}.Update(db)
}

func Delete(db *gorm.DB, inputSchematics []int) {
	schematics := Get(db, inputSchematics)
	for _, schematic := range *schematics {
		schematic.Delete(db)
	}
}
