package tier

import (
	"log"

	functionset "example.com/database/functionset"

	"gorm.io/gorm"
)

type Tier struct {
	gorm.Model
	FunctionSets []functionset.FunctionSet `gorm:"many2many:tier_functionset_associations" json:"function_sets"`
}

func (tier Tier) Create(db *gorm.DB) error {
	result := db.Create(&tier)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (tier *Tier) getAssociations(db *gorm.DB) {
	db.Model(&tier).Association("FunctionSets").Find(&tier.FunctionSets)
}

func (tier *Tier) Get(db *gorm.DB, inputTier int) {
	db.Where("id = ?", inputTier).Take(&tier)
	tier.getAssociations(db)
}

func (tier Tier) Update(db *gorm.DB) error {
	result := db.Save(&tier)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func (tier Tier) Delete(db *gorm.DB) error {
	result := db.Delete(&tier)
	if result.Error != nil {
		return result.Error
	}
	return nil
}

func Create(db *gorm.DB, functionSets []int) error {
	return Tier{
		FunctionSets: *functionset.Get(db, functionSets),
	}.Create(db)
}

func Get(db *gorm.DB, inputTiers []int) *[]Tier {
	var outputTiers []Tier
	for _, inputTier := range inputTiers {
		var outputTier Tier
		outputTier.Get(db, inputTier)
		outputTiers = append(outputTiers, outputTier)
	}
	return &outputTiers
}

func GetAll(db *gorm.DB) *[]Tier {
	var outputTierIDs []int
	result := db.Find(&outputTierIDs)
	if result.Error != nil {
		log.Println(result.Error)
	}
	return Get(db, outputTierIDs)
}

func Update(db *gorm.DB, id int, functionSets []int) error {
	outputTier := Tier{
		FunctionSets: *functionset.Get(db, functionSets),
	}
	outputTier.ID = uint(id)
	return outputTier.Update(db)
}

func Delete(db *gorm.DB, inputTiers []int) {
	tiers := Get(db, inputTiers)
	for _, tier := range *tiers {
		tier.Delete(db)
	}
}
